from src.model.fsm import *
import typer

app = typer.Typer()


def validate_helper(file: str):
    with open(file, "r") as f:
        data = f.read()
    try:
        fsm = FiniteStateMachine.from_json(data)
        fsm.validate()
        typer.echo(f"FSM {file} is valid")
        return fsm
    except Exception as e:
        typer.echo(f"FSM {file} is not valid: {e}")
        raise typer.Exit(code=1)


@app.command()
def convert(input: str, output: str = None):
    fsm = validate_helper(input)
    verilog = fsm.to_verilog()
    if output is None:
        typer.echo(verilog)
    else:
        with open(output, "w") as f:
            f.write(verilog)


@app.command()
def validate(input: str):
    validate_helper(input)


def main():
    app()


if __name__ == "__main__":
    main()



from __future__ import annotations
from dataclasses import dataclass
import json
from dacite import from_dict
import jinja2


@dataclass
class Register:
    name: str
    length: int


@dataclass
class Trasition:
    destination: str
    condition: str


@dataclass
class State:
    name: str
    transitions: list[Trasition]
    outputs: dict[str, int]


@dataclass
class FiniteStateMachine:
    TEMPLATE_PATH = "src/template.v"
    inputs: list[Register]
    outputs: list[Register]
    states: list[State]
    initial_state: str

    @staticmethod
    def from_json(data: str) -> FiniteStateMachine:
        return from_dict(data_class=FiniteStateMachine, data=json.loads(data))

    def validate(self):
        if self.initial_state not in [state.name for state in self.states]:
            raise Exception(
                f"Initial state {self.initial_state} not found in states")
        for state in self.states:
            for transition in state.transitions:
                if transition.destination not in [state.name for state in self.states]:
                    raise Exception(
                        f"Transition destination {transition.destination} not found in states")
                # TODO: validate condition
            for output in state.outputs:
                if output not in [register.name for register in self.outputs]:
                    raise Exception(f"Output {output} not found in outputs")
                register = [
                    register for register in self.outputs if register.name == output][0]
                if state.outputs[output] >= 2**register.length:
                    raise Exception(
                        f"Output {output} value {state.outputs[output]} is greater than register length {register.length}")

    def to_verilog(self) -> str:
        with open(self.TEMPLATE_PATH, "r") as f:
            template = jinja2.Template(f.read())
        import math
        template.globals["log2"] = math.log2
        template.globals["ceil"] = math.ceil
        return template.render(fsm=self)
